;------------------------------------------------;
;       Macro definition for kxDialog            ;
;       Published under the terms of GPL         ;
;       Written by Frank Sommer aka derPENGUIN   ;
;       Compile with FASM for Menuet                       ;
;       Version 0.15                             ;
;------------------------------------------------;

; defined widget types: (defined in kxData.inc)
; 0     kxWindwo (Main window, only one allowed)
; 10    kxLabel
; 12    kxULabel
; 20    kxBevel
; 50    kxButton
; 60    kxCheckbox


; TODO:
; * caption length
; * types should be resolved automatically


; This is the base structur,
; all widgets will inhertit from it.
; Do not use it directly
;-------------------------------------------------------------------------------
    struct kxWidget
      dtype	    dw	       ?	; Offset
      res1	    dw	       ?	; +2
      caption	    dd	       ?	; +4
      xpos	    dw	       0	; +8
      ypos	    dw	       0	; +10
      res2	    dw	       ?
      res3	    dw	       ?
      ccolor	    dd	       ?	; +16
    ends

; The sized base widget
; Its seperated from kxWidget for memory safe
; because kxLabel etc. needs no xsize and ysize.
; Do not use it directly
;-------------------------------------------------------------------------------
    struct kxSizedWidget kxWidget
      xsize	    dd	       ?	; +20
      ysize	    dd	       ?	; +24
      bcolor	    dd	       ?	; +28
    ends

; Window, kxType 0
; inherits from kxSizedWidget
    struct kxWindow kxSizedWidget
      gcolor	    dd	       ?	; +32   grab bar color
      fcolor	    dd	       ?	; +36   frame color
    ends

; Label, kxType 10
; inherits from kxWidget
    struct kxLabel kxWidget
; no extra proerties
    ends

; underlined Label, kxType 12
; inherits from kxLabel
    struct kxULabel kxLabel
      reserved1     dw	       ?	; +20
      reserved2     dw	       ?	; +22
    ends

; Bevel,kxType 20
; inherits from kxSizedWidget
    struct kxBevel kxSizedWidget
      bsize	dw	   ?		; +32   border size
      reserved2 dw	   ?		; +34
      bstyle	dd	   ?		; +36   bevel style,
					;       bsInner or bsOuter
    ends

; Button,kxType 50,  atm expands only id
; inherits from kxSizedWidget
    struct kxButton kxSizedWidget

      focustab	    dw	       ?	; +32   bit 16 1/0 can get focus
					;       bit 15 1/0 has focus
					;       bit 14-0 tab order, must be unique
      id	    dw	       ?	; +34
    ends

; CheckBox,kxType 60
; inherits from kxButton
    struct kxCheckbox kxButton
      checked	    dd	       ?	; +36   should be only 0 or 0xffffff
    ends

; ListBox,kxType 70
; inherits from kxButton
    struct kxListbox kxButton
      checked	    dd	       ?	; not used atm
    ends

; kxSlider kxType 90
; inherits from kxButton
    struct kxSlider kxButton
      min	 dw	       ?	 ; +40  mininmum
      max	 dw	       ?	 ; +42  maximum
    ends

; ============ Dreate a new widget ===============
macro kxNew type,name {
  ; we create an instance and set type number
  ; better would be "case"
  if <type> eq <kxWindow>
      name kxWindow _kxWindow
  endf

  if <type> eq <kxLabel>
      name kxLabel _kxLabel
  endf

  if <type> eq <kxULabel>
      name kxULabel _kxULabel
  endf

  if <type> eq <kxBevel>
      name kxBevel _kxBevel
  endf

  if <type> eq <kxButton>
      name kxButton _kxButton
  endf

  if <type> eq <kxCheckbox>
      name kxCheckbox _kxCheckbox
  endf

  if <type> eq <kxListbox>
      name kxListbox _kxListbox;,,,0,0,0,50,50,0
  endf

  mov edi, name
  call _kx_register_w	     ; register widget in event handler

}

; ============ Draw the widget ===============
macro kxDraw name,type,strlen {
  pusha
  local n,text,xp,yp,xs,ys,cc,bc,bs,cl,ck
  cl=strlen
  text=n+4
  xp=n+8
  yp=n+10
  xs=n+20
  ys=n+24
  cc=n+16
  bc=n+28
  n=name
;-------------------------------------------------------------------------------
; draw window
  if <type> eq <_kxWindow>
      mcall 0,<[xp],[xs]>,<[yp],[ys]>,[bc]	    ; the window itself
      mcall 4, <8,8>,[cc],[text],cl		     ; its title
  endf
;-------------------------------------------------------------------------------
; draw label
  if <type> eq <_kxLabel>
      mcall 4, <[xp],[yp]>,[cc],[text],cl	; the label
  endf
;-------------------------------------------------------------------------------
; draw underlined label
  if <type> eq <_kxULabel>
      mcall 4, <[xp],[yp]>,[cc],[text],cl	; the label
      mov eax,cl	    ; string length
      mov edx,[cc]	    ; color
      mov ebx,[xp]
      shl ebx,16
      add ebx,[xs]
      mov ecx,[yp]
      shl ecx,16
      add ecx,[ys]
      call _kx_uline
  endf
;-------------------------------------------------------------------------------
; draw bevel
  if <type> eq <_kxBevel>
      bs=n+36
;      ; we switch the border colors for raised or lowered 3D effect
      mov esi,[n+32]	    ; border width
      mov edx,[bs]	    ; style
      mov ebx,[xp]
      shl ebx,16
      add ebx,[xs]
      mov ecx,[yp]
      shl ecx,16
      add ecx,[ys]
      call _kx_rect
  endf
;-------------------------------------------------------------------------------
; draw button
  if <type> eq <_kxButton>
; do we have the focus? so draw a pointed rectangle
; this should be writen as function
      mcall 8,<[xp],[xs]>,<[yp],[ys]>,[n+32],[bc]   ; the button itself
      ; center the caption:
      push dword [xp] dword [yp]
      mov eax,[xs]
      shr eax,1
      sub eax,cl*3 ; the count of subs depends on font size, 3 is good if fsBold
      add [xp],eax
      mov eax,[ys]
      shr eax,1
      sub eax,2
      add [yp],eax
      mcall 4, <[xp],[yp]>,[cc],[text],cl	; its caption
      pop dword  [yp] dword [xp]

      mov eax,[n+32]
      shl eax,17
      cmp eax, 0
      je @f
      mov bx,word [xp]
      add ebx,3
      shl ebx,16
      add ebx,[xs]
      sub ebx,6
      mov cx,word [yp]
      add ecx,3
      shl ecx,16
      add ecx,[ys]
      sub ecx,6
      call _kx_set_focus
    @@:

  endf
;-------------------------------------------------------------------------------
; draw checkbox
  if <type> eq <_kxCheckbox>
; do we have the focus? so draw a pointed rectangle
; this should be writen as function
      mov eax,[n+32]
      shl eax,17
      cmp eax,0
      je @f
      mov bx,word [xp]
      sub ebx,3
      shl ebx,16
      add ebx,cl*6+15
      add ebx,6
      mov cx,word [yp]
      sub ecx,3
      shl ecx,16
      add ecx,[ys]
      add ecx,6
      call _kx_set_focus
    @@:
  ; we use a non visible button for click handling
      mov edx,1
      shl edx,30	  ; set bit 30
      or  edx,[n+32]
      mcall 8,<[xp],cl*6+15>,<[yp],10>

      ck=n+36
      ; the size of checkboxes is always 10
      mov eax,10
      mov [xs],eax
      mov [ys],eax
      mov esi,0
      ; we switch the border colors for checked status
      mov edx,[ck]
      xor edx,TRUE
      mov ebx,[xp]
      shl ebx,16
      add ebx,[xs]
      mov ecx,[yp]
      shl ecx,16
      add ecx,[ys]
      call _kx_rect		 ; in gfunc.inc
      push dword [xp]
      push dword [yp]
      mov eax,[xp]			       ; place caption 15 pts left from box
      add eax, 15
      mov [xp],eax
      mov eax,[yp]			       ; place caption 2 pts down
      add eax, 2
      mov [yp],eax
      mcall 4, <[xp],[yp]>,[cc],[n+4],cl       ; its caption
      pop dword [yp]
      pop dword [xp]
;mcall 47,4*65536+10,[xp],50*65536+360,0
;mcall 47,4*65536+10,[yp],50*65536+370,0
;mcall 47,4*65536+10,[xs],50*65536+380,0
;mcall 47,4*65536+10,[ys],50*65536+390,0
  endf
;-------------------------------------------------------------------------------
; draw listbox
  if <type> eq <_kxListbox>
      mov edx,[bc]
      mov ebx,[xp]
      shl ebx,16
      add ebx,[xs]
      mov ecx,[yp]
      shl ecx,16
      add ecx,14	   ; [ys]
      call _kx_lb	       ; in gfunc.inc
  endf

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
   popa 		    ; end of macro, dont delete
}



; ============ Draw a message box ===============
;macro MsgBox text,strlen {
; do nothing atm
;}

;dbox:
;ret
;jmp wait_event